class MusicTags:
    @staticmethod
    def get(tags) -> dict:
        return {
            "album": tags[0].decode(),
            "albumArtist": tags[1].decode(),
            "bitrate": int(tags[2].decode()),
            "composers": tags[3].decode(),
            "conductors": tags[4].decode(),
            "duration": int(tags[5].decode()),
            "genre": tags[6].decode(),
            "producers": tags[7].decode(),
            "publisher": tags[8].decode(),
            "rating": int(tags[9].decode()),
            "subtitle": tags[10].decode(),
            "title": tags[11].decode(),
            "trackNumber": int(tags[12].decode()),
            "writers": tags[13].decode(),
            "year": int(tags[14].decode()),
        }

class VideoTags:
    @staticmethod
    def get(tags):
        return {
            "bitrate": tags[0].decode(),
            "directors": tags[1].decode(),
            "duration": tags[2].decode(),
            "height": tags[3].decode(),
            "keywords": tags[4].decode(),
            "longitude": tags[5].decode(),
            "latitude": tags[6].decode(),
            "orientation": tags[7].decode(),
            "producers": tags[8].decode(),
            "rating": tags[9].decode(),
            "subtitle": tags[10].decode(),
            "title": tags[11].decode(),
            "width": tags[12].decode(),
            "writers": tags[13].decode(),
            "year": tags[14].decode(),
        }



import os
from typing import Callable

from . import Media, Player
from .library import lib
from ctypes import c_int32, POINTER, CFUNCTYPE, c_char_p


class NativeControlsStatus:
    Closed = 0
    Changing = 1
    Stopped = 2
    Playing = 3
    Paused = 4


class NativeControlsButton:
    Play = 0
    Pause = 1
    Stop = 2
    Record = 3
    FastForward = 4
    Rewind = 5
    Next = 6
    Previous = 7
    ChannelUp = 8
    ChannelDown = 9


class NativeControls:
    def __init__(self, player: Player):
        self.player = player
        self._callbacks = []

    def create(self, callback: Callable[[int], None]) -> None:
        cb = CFUNCTYPE(None, c_int32)(callback)
        self._callbacks.append(cb)
        lib.PlayerNativeControlsCreate(self.player.id, cb)

    def create_callback(self) -> Callable[[Callable[[int], None]], None]:
        def wrapper(callback: Callable[[int], None]) -> None:
            self.create(callback)

        return wrapper

    def set_status(self, status: int):
        lib.PlayerNativeControlsSetStatus(self.player.id, status)

    def update(self, media: Media):
        folder = os.path.dirname(__file__)
        file = "thumbnail.png"
        thumb = os.path.join(folder, file)
        media.extract_thumbnail(folder, file)

        lib.PlayerNativeControlsUpdate.argtypes = [
            c_int32,
            c_int32,
            POINTER(c_char_p),
            c_char_p,
        ]

        meta = media.tags_from_music()

        metalist = [
            meta["albumArtist"].encode("utf-8"),
            meta["title"].encode("utf-8"),
            "1".encode("utf-8"),
            meta["publisher"].encode("utf-8"),
            meta["title"].encode("utf-8"),
            str(meta["trackNumber"]).encode("utf-8"),
        ]

        metas = (c_char_p * len(metalist))(*metalist)

        lib.PlayerNativeControlsUpdate(self.player.id, 1, metas, thumb)

    def clear(self):
        lib.PlayerNativeControlsClear(self.player.id)

    def dispose(self):
        lib.PlayerNativeControlsDispose(self.player.id)



import platform
import ctypes.util
import os
import ctypes

if platform.system() == "Linux":
    lib_name = "libwinmedia.so"
elif platform.system() == "Windows":
    lib_name = "libwinmedia.dll"


def find_lib() -> ctypes.CDLL:
    """Search for the library.

    You can place the DLL somewhere in %PATH% or set
    the environment variable "LIBWINMEDIA_PATH".

    Raises:
        OSError: The "LIBWINMEDIA_PATH" env variable is invalid.
        OSError: libwinmedia.dll is not found.

    Returns:
        ctypes.CDLL: An instance of the DLL.
    """

    # search in env variables
    if "LIBWINMEDIA_PATH" in os.environ:
        try:
            lib_path = os.path.join(os.environ["LIBWINMEDIA_PATH"], lib_name)
            return ctypes.CDLL(lib_path)
        except OSError:
            raise OSError("Invalid LIBWINMEDIA_PATH specified. Please fix.")

    # search in PATH
    try:
        path = ctypes.util.find_library(lib_name)
        if not path:
            raise FileNotFoundError
        return ctypes.CDLL(path)
    except FileNotFoundError:
        # not found
        raise OSError(
            "Cannot find libwinmedia.dll in your system %PATH%. One way to deal with this is to "
            "ship libwinmedia.dll with your script and put the directory where your script is "
            'located in %PATH% before "import libwinmedia": '
            'os.environ["PATH"] = os.path.dirname(__file__) + os.pathsep + os.environ["PATH"] '
            'If libwinmedia.dll is located elsewhere, you can add that path to os.environ["PATH"].'
        )


lib = find_lib()



from typing import List

from .media import Media


class Playlist:
    def __init__(self, *medias: Media):
        self.medias = list(medias)

    def add(self, media) -> None:
        self.medias.append(media)

    def remove(self, media) -> None:
        self.medias.remove(media)

    def delete(self, i: int) -> None:
        self.medias.pop(i)

    @property
    def uris(self) -> List[str]:
        uris = []
        for media in self.medias:
            uris.append(media.uri)
        return uris

    @property
    def ids(self) -> List[int]:
        ids = []
        for media in self.medias:
            ids.append(media.id)
        return ids

    @property
    def length(self) -> int:
        return len(self.medias)



import platform
import time
from threading import Thread

from .playlist import Playlist
from .media import Media
from .library import lib
from typing import Callable, Union, Optional

from ctypes import CFUNCTYPE, c_bool, c_int32, c_float, c_char_p

player_id = 0


class Player(object):
    """A class for controlling a media player."""

    def __init__(self, show_video: Optional[bool] = False, sleep: Optional[int] = 1):
        """Create a new [Player] instance.

        This function creates another thread on Linux and adds a little delay on Linux for avoiding Python to be faster than C++

        Args:
            show_video (bool, optional): Whether to show the video window. Defaults to False.
            sleep (int, optional): Sleep after creating player instance. Defaults to 1.
        """

        global player_id
        self.id = player_id

        if platform.system() == "Linux":
            """
            Here we import PyGObject, which basically handles all kinds of GTK stuff
            We need this, otherwise nothing would work... :sweat_smile:
            I don't want this API to be blocking, so I decided to make it a little bit thready...
            Now, this creates a thread, which basically handles all GTK operations (AFAIK)
            So, this makes this API non-blocking.
            This is executed only on Linux OS
            """
            import gi
            gi.require_version("Gtk", "3.0")
            self.gtkthread = Thread(target=Player.__runGTK)
            self.gtkthread.start()

        lib.PlayerCreate(self.id, show_video)
        player_id += 1

        # to prevent callbacks from being garbage collected
        self._callbacks = []

        if platform.system() == "Linux":
            """
            Delay ensures object is inserted into the std::unordered_map before calling any Player attribute.
            @alexmercerind will ensure the initialization in C++ itself in future & make this stuff sync.
            I included this directly into function, since this is easiest for user...
            """
            time.sleep(sleep)

    @staticmethod
    def __runGTK():
        from gi.repository import Gtk
        Gtk.main()

    def open(self, media: Union[Media, Playlist], autostart: bool = True) -> None:
        """Provide a [Media] or [Playlist] instance to the [Player].

        Args:
            media ([Media] or [Playlist]): A media file or a Playlist.
            autostart (bool, optional): Whether to autostart playback of the provided media. Defaults to True.
        """

        if type(media) is Media:
            playlist = Playlist()
            playlist.add(media)
        else:
            playlist = media

        urislist = []
        for i in playlist.uris:
            urislist.append(i.encode("utf-8"))
        uris = (c_char_p * len(urislist))(*urislist)
        ids = (c_int32 * len(playlist.ids))(*playlist.ids)

        lib.PlayerOpen(self.id, len(playlist.medias), uris, ids)
        if autostart:
            self.play()

    def play(self) -> None:
        """Start playing the media that is currently open in the player."""

        lib.PlayerPlay(self.id)

    def pause(self) -> None:
        """Pause the playback of the current media."""

        lib.PlayerPause(self.id)

    def next(self) -> None:
        lib.PlayerNext(self.id)

    def back(self) -> None:
        lib.PlayerBack(self.id)

    def jump(self, id: int) -> None:
        lib.PlayerJump(self.id, id)

    def dispose(self) -> None:
        """Release system resources and kill the [Player] instance."""

        self.close_window()
        lib.PlayerDispose(self.id)
        self._callbacks.clear()

    def close_window(self) -> None:
        """Close the video player window."""

        lib.PlayerCloseWindow(self.id)

    @property
    def looping(self) -> bool:
        return lib.PlayerIsLooping(self.id)

    @looping.setter
    def looping(self, value: bool) -> None:
        lib.PlayerSetIsLooping(self.id, value)

    @property
    def autoplay(self) -> bool:
        return lib.PlayerIsAutoplay(self.id)

    @autoplay.setter
    def autoplay(self, value: bool) -> None:
        lib.PlayerSetAutoplay(self.id, value)

    @property
    def audio_balance(self) -> float:
        lib.PlayerGetAudioBalance.restype = c_float
        return lib.PlayerGetAudioBalance(self.id) * 100

    @audio_balance.setter
    def audio_balance(self, value: float) -> None:
        lib.PlayerSetVolume.argtypes = [c_int32, c_float]
        lib.PlayerSetAudioBalance(self.id, value / 100)

    @property
    def rate(self) -> float:
        lib.PlayerGetRate.restype = c_float
        return lib.PlayerGetRate(self.id) * 100

    @rate.setter
    def rate(self, value: float) -> None:
        lib.PlayerSetRate.argtypes = [c_int32, c_float]
        lib.PlayerSetRate(self.id, value / 100)

    @property
    def volume(self) -> float:
        lib.PlayerGetVolume.restype = c_float
        return lib.PlayerGetVolume(self.id) * 100

    @volume.setter
    def volume(self, value: int) -> None:
        lib.PlayerSetVolume.argtypes = [c_int32, c_float]
        lib.PlayerSetVolume(self.id, value / 100)

    @property
    def position(self) -> int:
        return lib.PlayerGetPosition(self.id)

    @position.setter
    def position(self, value: int) -> None:
        lib.PlayerSeek(self.id, value)

    def seek(self, value: int) -> None:
        self.position = value

    def set_playing_callback(self, callback: Callable[[bool], None]) -> None:
        cb = CFUNCTYPE(None, c_bool)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetIsPlayingEventHandler(self.id, cb)

    def playing_callback(self) -> Callable[[Callable[[bool], None]], None]:
        def wrapper(callback: Callable[[bool], None]) -> None:
            self.set_playing_callback(callback)

        return wrapper

    def set_completed_callback(self, callback: Callable[[bool], None]) -> None:
        cb = CFUNCTYPE(None, c_bool)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetIsCompletedEventHandler(self.id, cb)

    def completed_callback(self) -> Callable[[Callable[[bool], None]], None]:
        def wrapper(callback: Callable[[bool], None]) -> None:
            self.set_completed_callback(callback)

        return wrapper

    def set_buffering_callback(self, callback: Callable[[bool], None]) -> None:
        cb = CFUNCTYPE(None, c_bool)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetIsBufferingEventHandler(self.id, cb)

    def buffering_callback(self) -> Callable[[Callable[[bool], None]], None]:
        def wrapper(callback: Callable[[bool], None]) -> None:
            self.set_buffering_callback(callback)

        return wrapper

    def set_volume_callback(self, callback: Callable[[float], None]) -> None:
        cb = CFUNCTYPE(None, c_float)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetVolumeEventHandler(self.id, cb)

    def volume_callback(self) -> Callable[[Callable[[float], None]], None]:
        def wrapper(callback: Callable[[float], None]) -> None:
            self.set_volume_callback(callback)

        return wrapper

    def set_rate_callback(self, callback: Callable[[float], None]) -> None:
        cb = CFUNCTYPE(None, c_float)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetRateEventHandler(self.id, cb)

    def rate_callback(self) -> Callable[[Callable[[float], None]], None]:
        def wrapper(callback: Callable[[float], None]) -> None:
            self.set_rate_callback(callback)

        return wrapper

    def set_position_callback(self, callback: Callable[[int], None]) -> None:
        cb = CFUNCTYPE(None, c_int32)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetPositionEventHandler(self.id, cb)

    def position_callback(self) -> Callable[[Callable[[int], None]], None]:
        def wrapper(callback: Callable[[int], None]) -> None:
            self.set_position_callback(callback)

        return wrapper

    def set_duration_callback(self, callback: Callable[[int], None]) -> None:
        cb = CFUNCTYPE(None, c_int32)(callback)
        self._callbacks.append(cb)
        lib.PlayerSetDurationEventHandler(self.id, cb)

    def duration_callback(self) -> Callable[[Callable[[int], None]], None]:
        def wrapper(callback: Callable[[int], None]) -> None:
            self.set_duration_callback(callback)

        return wrapper



"""Python bindings for libwinmedia.

Use native Windows APIs to play local files and URLs,
control playback, extract metadata, and show native controls.
"""

from .player import Player
from .media import Media
from .nativecontrols import NativeControls, NativeControlsStatus
from .playlist import Playlist

__author__ = "libwinmedia Team"
__version__ = "1.0.0"
__email__ = "mytja@protonmail.com"



from setuptools import setup

setup(name="libwinmedia",
      version="1.0.1",
      description="Python bindings for libwinmedia, a tiny yet powerful media playback library for Windows and Linux.",
      long_description=open("README.md", "r").read(),
      long_description_content_type="text/markdown",
      author="libwinmedia Team",
      author_email="mytja@protonmail.com",
      url="https://github.com/libwinmedia/libwinmedia-py",
      include_package_data=True,
      license="MIT",
      packages=["libwinmedia"],
      package_dir={
          "libwinmedia": "libwinmedia",
      },
      classifiers=[
          "Topic :: Multimedia :: Sound/Audio",
          "Operating System :: POSIX :: Linux",
          "Operating System :: Microsoft :: Windows :: Windows 10",
          "Operating System :: Microsoft :: Windows",
          "Topic :: Multimedia :: Video",
          "License :: OSI Approved :: MIT License"
      ],
      install_requires=[
            'PyGObject; platform_system=="Linux"'
      ],
      data_files=[]
)



import os
import platform

from .library import lib
from ctypes import POINTER, c_char_p
from .tags import MusicTags, VideoTags

media_id = 0


class Media(object):
    """A class representing a media file."""

    def __init__(self, uri: str, parse: bool = True):
        """Create a new Media instance.

        The URI can be either a local file (e.g. "file://C:/music/track.mp3")
        or an HTTP URL (e.g. "https://www.kozco.com/tech/piano2.wav").

        Args:
            uri (str): A URI of the media.
            parse (bool, optional): Whether to parse the media. Defaults to True. True is required for duration parsing
        """

        global media_id
        self.id = media_id
        if not os.path.isabs(uri) and not ("https" in uri or "http" in uri):
            uri = os.path.join(os.path.dirname(os.path.abspath(uri)), uri)
        self.uri = uri
        if platform.system() != "Linux":
            lib.MediaCreate(self.id, uri.encode("utf-8"), parse)
        media_id += 1

    def dispose(self) -> None:
        """Release system resources and kill the media instance."""

        lib.MediaDispose(self.id)

    @property
    def duration(self) -> int:
        return lib.MediaGetDuration(self.id)

    def tags_from_music(self) -> dict:
        # TODO: add docstring
        lib.TagsFromMusic.args = [c_char_p]
        lib.TagsFromMusic.restype = POINTER(c_char_p)

        meta = lib.TagsFromMusic(self.uri.encode("utf-8"))
        return MusicTags.get(meta)

    def tags_from_video(self) -> dict:
        lib.TagsFromVideo.args = [c_char_p]
        lib.TagsFromVideo.restype = POINTER(c_char_p)

        meta = lib.TagsFromVideo(self.uri.encode("utf-8"))
        return VideoTags.get(meta)

    def extract_thumbnail(self, output_folder: str, output_file: str) -> None:
        """Extract the thumbnail and save it to a file.

        Args:
            output_folder (str): A folder for saving the thumbnail.
            output_file (str): A name of the thumbnail file.
        """
        lib.TagsExtractThumbnail(self.uri.encode(), ("file://" + output_folder).encode(), output_file.encode(), 2, 400)
